from fastapi import APIRouter, Depends, Query
from sqlalchemy.orm import Session
from sqlalchemy.orm import joinedload
from backend.database import get_session
from backend.models.models import Shop, Product, Category
from typing import List, Union

router = APIRouter()


@router.get("/search", response_model=List[Union[Shop, Product]])
def search(
    name: str = Query(None, description="Name to search"),
    category: str = Query(None, description="Category to filter by (for products)"),
    search_type: str = Query(
        "both", description="Search type: 'shops', 'products', or 'both'"
    ),
    db: Session = Depends(get_session),
):
    results = []
    if search_type in ["shops", "both"]:
        shop_query = db.query(Shop)
        if name:
            shop_query = shop_query.filter(Shop.name.ilike(f"%{name}%"))
        results.extend(shop_query.all())

    if search_type in ["products", "both"]:
        product_query = db.query(Product).options(joinedload(Product.category))
        if name:
            product_query = product_query.filter(Product.name.ilike(f"%{name}%"))
        if category:
            product_query = product_query.join(Category).filter(
                Category.name.ilike(f"%{category}%")
            )
        results.extend(product_query.all())

    return results
